﻿module ObjReadForm
open System
open System.Collections.Generic
open System.Threading
open System.Drawing
open System.IO
open OpenTK
open OpenTK.Graphics
open OpenTK.Graphics.OpenGL
open Common
open ObjRead
open CgNet.GL
type ObjReadForm() =
    inherit GameWindow(1400,900,new GraphicsMode(new ColorFormat(16),16))
    do base.VSync <- VSyncMode.On
    let caram = UICamera()
    let cgContext = new CgContext()
    let sphere = Sphere(0.2f,3)
    let sphere1 = new Sphere(0.2f,3)
    let mutable lightPosition = Vector3(5.f,20.f,5.f)// [|0.f;5.f;8.f|]
    let modelPath = Environment.CurrentDirectory;
    let model = new ObjModel(modelPath + "\Data\Models\Sherry\sherry_sailor_blue_re6.obj")
    //let model = new ObjModel(modelPath + "\Data\Models\Tails.obj")
    override v.OnLoad e =
        base.OnLoad(e)
        GL.ClearColor(Color.MidnightBlue)
        GL.Enable(EnableCap.DepthTest)
        GL.EnableClientState(ArrayCap.VertexArray)
        GL.EnableClientState(ArrayCap.TextureCoordArray) 
        GL.Enable(EnableCap.Texture2D)  
        model.LoadObjModel()
        model.CreateVbo()
        cgContext.CreateVectexProgram(modelPath + @"\Data\Shaders\v1.cg","v_main")
        cgContext.CreateFragmentProgram(modelPath + @"\Data\Shaders\f1.cg","f_main")
        cgContext.FragmentParameter("globalAmbient").Set([|0.9f; 0.9f; 0.9f|])//)Vector3.Zero
        cgContext.FragmentParameter("lightColor").Set([|0.95f;0.95f;0.95f|])
        caram.Transelt(3.,5.,-20.) 
    override this.OnRenderFrame(e) =
        base.OnRenderFrame e 
        GL.Clear (ClearBufferMask.ColorBufferBit ||| ClearBufferMask.DepthBufferBit)
        let mutable v = Matrix4.LookAt(caram.Eye,caram.Target,Vector3.UnitY)
        model.Translation <- Vector3(5.f,0.f,0.f)
        model.Rotate <- Vector3(1.0f,0.f,0.f)
        model.RotateAngle <- float32 (-Math.PI/2.0)
        let m = model.ModelMatrix
        m.Transpose()
        v.Transpose()
        let mv = Matrix4.Mult(v,m)
        cgContext.EnableProfile()
        let mutable p = Matrix4.Identity
        GL.GetFloat(GetPName.ProjectionMatrix,&p)
        p.Transpose()
        let mvp = Matrix4.Mult(p,mv)
        cgContext.VertexParameter("mvp").SetMatrix(MatrixToArray1 mvp)
        //从世界坐标系的位置转化成灯光的位置 
        let modelEye = Vector3.Transform(caram.Eye,model.InvertMatrix)
        cgContext.FragmentParameter("eyePosition").Set(modelEye)
        let modelLight = Vector3.Transform(lightPosition,model.InvertMatrix)
        cgContext.FragmentParameter("lightPosition").Set(modelLight)
        model.Groups |> List.iteri (fun i p ->             
            cgContext.FragmentParameter("Ke").Set(p.Material.Emissive)
            cgContext.FragmentParameter("Ka").Set(p.Material.Ambient)
            cgContext.FragmentParameter("Kd").Set(p.Material.Diffuse)
            cgContext.FragmentParameter("Ks").Set(p.Material.Specular)
            cgContext.FragmentParameter("shininess").Set(p.Material.Shiness)
            cgContext.FragmentParameter("dtext").SetTexture(p.Material.DiffuseID)
            cgContext.FragmentParameter("dtext").EnableTexture()
            cgContext.FragmentParameter("maptext").SetTexture(p.Material.BumpID)
            cgContext.FragmentParameter("maptext").EnableTexture()
            cgContext.UpdateParameter()
            p.DrawVBO()
            cgContext.FragmentParameter("dtext").DisableTexture()
            cgContext.FragmentParameter("maptext").DisableTexture()
            )       
        let m = Matrix4.CreateTranslation(lightPosition)
        m.Transpose()
        let mv = Matrix4.Mult(v,m)
        let mvp = Matrix4.Mult(p,mv)
        cgContext.VertexParameter("mvp").SetMatrix(MatrixToArray1 mvp) 
        cgContext.FragmentParameter("lightPosition").Set([|0;0;-3|])
        cgContext.FragmentParameter("Ka").Set([|0.9f;0.9f;0.9f|])
        cgContext.FragmentParameter("Kd").Set([|0.9f;0.9f;0.9f|])
        cgContext.FragmentParameter("maptext").SetTexture(0)
        cgContext.FragmentParameter("dtext").SetTexture(0)
        cgContext.UpdateParameter()
        sphere.Draw()
        let mvp = Matrix4.Mult(p,v)
        cgContext.VertexParameter("mvp").SetMatrix(MatrixToArray1 mvp)
        cgContext.FragmentParameter("lightPosition").Set([|3;3;3|]) 
        cgContext.UpdateParameter()
        sphere1.Draw()
        GL.Begin BeginMode.Lines
        GL.Color3(Color.Black)
        GL.Vertex3(Vector3.Zero)
        GL.Vertex3(Vector3.UnitX * 1000.f)
        GL.Color3(Color.White)
        GL.Vertex3(Vector3.Zero)
        GL.Vertex3(Vector3.UnitY * 1000.f)
        GL.Color3(Color.Red)
        GL.Vertex3(Vector3.Zero)
        GL.Vertex3(Vector3.UnitZ * 1000.f)
        GL.End()
        cgContext.DisableProfile()
        this.SwapBuffers()
    override v.OnResize e =
        base.OnResize e
        GL.Viewport(0,0,v.Width,v.Height)
        let ratio = float32 v.Width/float32 v.Height
        let mutable pective = Matrix4.CreatePerspectiveFieldOfView(float32 MathHelper.PiOver4,ratio,0.1f,1000.0f)
        GL.MatrixMode(MatrixMode.Projection)
        GL.LoadMatrix(&pective)
    override v.OnUpdateFrame e =
        base.OnUpdateFrame e
        caram.UIChange(v.Keyboard,v.Mouse)
    override v.OnUnload e =
        base.OnUnload(e)
        cgContext.Unload()
        